import json

import requests


def get_data_as_file(url, filename):
    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        with open(filename, "wb") as f:
            for chunk in r.iter_content(chunk_size=8192):
                if chunk:
                    f.write(chunk)


def get_data_as_json(url):
    response = requests.get(url)
    return response.json()


def get_data_as_text(url):
    response = requests.get(url)
    return response.text


def get_data_as_stream(url):
    return requests.get(url, stream=True)


def convert_stream_to_json(r):
    r.iter_content(1000000000)
    return json.loads(r.content)


def convert_stream_to_file(r, local_filename):
    f = open(local_filename, "wb")
    for chunk in r.iter_content(chunk_size=512 * 1024):
        if chunk:
            f.write(chunk)
    f.close()



config = {}


def set_config(conf):
    global config
    config = conf


def get_config():
    global config
    return config


def get_full_url(relative_url):
    return "%s/api/%s/%s" % (config["HOST"], config["API_VERSION"], relative_url)


def get_headers():
    return {
        "Accept": "application/json",
        "Authorization": "JWT %s" % (config["TOKEN"]),
        "Content-Type": "application/json",
    }



import argparse
import datetime
import sys
import threading
import time
import traceback

from deepdiff import DeepDiff
from sentry_sdk import capture_exception, flush, init

from . import api
from .analyzer import Analyzer
from .globals import set_config


def flush_print(line):
    print(datetime.datetime.now().isoformat(), line)
    sys.stdout.flush()


def flush_write(message):
    sys.stdout.write(message)
    sys.stdout.flush()


def spinning_cursor():
    while True:
        for cursor in "▁▂▃▄▅▆▇█▇▆▅▄▃▂▁":
            yield cursor


class Coordinator:
    def __init__(self, converter, path: str):
        self.analyzers = {}
        self.converter = converter
        self.path = path
        self.running = False
        self.parser = argparse.ArgumentParser(
            description="Create a Centroid Ambulation analysis for the BehaviorCloud platform."
        )
        self.parser.add_argument(
            "--host",
            nargs="?",
            required=True,
            help="The hostname of the BehaviorCloud api server. "
            + "Typically, this is api.behaviorcloud.com.",
        )
        self.parser.add_argument(
            "--token",
            nargs="?",
            required=True,
            help="The JWT token provided by the BehaviorCloud platform.",
        )
        self.parser.add_argument("--id", nargs="?", help="The queued analysis id.")
        self.parser.add_argument(
            "--daemon",
            action="store_true",
            help="Run this analyzer is a daemonized mode, watching for new jobs.",
        )
        self.parser.add_argument(
            "--runtests",
            action="store_true",
            help="Rerun public analyses comparing this analyzer's results to existing results.",
        )

    def analyze(self, queued_analysis_id):
        if queued_analysis_id in self.analyzers:
            return
        analyzer = Analyzer(queued_analysis_id, self.converter)
        self.analyzers[queued_analysis_id] = analyzer

        flush_print("Starting Converting Queued Analysis: %s" % (queued_analysis_id))
        try:
            analyzer.start()
            flush_print("Finished Converting Queued Analysis: %s" % (queued_analysis_id))
            return True
        except Exception:
            capture_exception()
            flush()
            flush_print("Error Converting Queued Analysis: %s" % (queued_analysis_id))
            traceback.print_exc()

        return False

    def run(self):
        arguments = self.parser.parse_args()
        set_config({"HOST": arguments.host, "TOKEN": arguments.token, "API_VERSION": "1.1"})

        init("https://5196a2999eb1497a8fd043d2683e30c0@sentry.io/1298461")

        if arguments.id:
            self.analyze(arguments.id)
        elif arguments.daemon:
            self.start_daemon()
        elif arguments.runtests:
            self.runtests()
        elif arguments.createtests:
            self.createtests()
        else:
            flush_print("You must supply an id when not using daemon or test mode")

    def start_daemon(self):
        if self.running:
            return
        self.running = True
        flush_print("Starting analysis daemon")
        t = threading.Thread(target=self.daemon)
        t.start()

    def daemon(self):
        spinner = spinning_cursor()

        while self.running:
            queued_analyses = api.get_queued_analysis_index(self.path)
            ran = False
            for queued_analysis in queued_analyses:
                if queued_analysis["ended"]:
                    continue
                id = queued_analysis["id"]
                ran = self.analyze(id)
            if not ran:
                for _ in range(15):
                    message = next(spinner) + " waiting for analyses..."
                    flush_write(message)
                    time.sleep(0.1)
                    [sys.stdout.write("\b") for x in message]

    def runtests(self):
        queued_analyses = api.get_queued_analysis_index(self.path, ended=True, public=True)
        flush_print("Found %s public %s analyses" % (len(queued_analyses), self.path))

        for queued_analysis in queued_analyses:
            id = queued_analysis["id"]
            analyzer = Analyzer(id, self.converter)
            result, old, new = analyzer.test()
            if result:
                flush_print("Passed: Queued Analysis %s" % (id))
            else:
                flush_print("Failed: Queued Analysis %s" % (id))
                flush_print(DeepDiff(old, new))
                sys.exit(1)

        sys.exit(0)

    def stop(self):
        self.running = False



import datetime
import json
import sys
import traceback
from sentry_sdk import capture_exception, flush
from . import api, data


def flush_print(line):
    print(datetime.datetime.now().isoformat(), line)
    sys.stdout.flush()


class Analyzer:
    def __init__(self, queued_analysis_id, converter):
        self.converter = converter
        self.queued_analysis_id = queued_analysis_id

    def test(self):
        queued_analysis = api.get_queued_analysis(self.queued_analysis_id)
        source = api.get_dataset(queued_analysis["source"])
        target_ids = queued_analysis["targets"]
        targets = [api.get_dataset(target_id) for target_id in target_ids]
        source_request = data.get_data_as_stream(source["url"])
        outputs = self.converter(
            source_request=source_request,
            source_settings=source["meta_data"],
            settings=queued_analysis["settings"],
            source=source,
            targets=targets,
        )

        old = data.get_data_as_json(targets[0]["url"])
        new = json.loads(outputs[0]["data"])

        return old == new, old, new

    def start(self):
        self.queued_analysis = api.get_queued_analysis(self.queued_analysis_id)
        self.source = api.get_dataset(self.queued_analysis["source"])
        self.target_ids = self.queued_analysis["targets"]
        self.targets = [api.get_dataset(target_id) for target_id in self.target_ids]
        api.queued_analysis_mark_started(self.queued_analysis_id)
        try:
            source_request = data.get_data_as_stream(self.source["url"])
            outputs = self.converter(
                source_request=source_request,
                source_settings=self.source["meta_data"],
                settings=self.queued_analysis["settings"],
                source=self.source,
                targets=self.targets,
            )
            for target in outputs:
                api.dataset_attach_data(target["id"], target["data"], target["extension"])
        except Exception as e:
            capture_exception(e)
            flush()
            flush_print('Error starting analysis %s' % (self.queued_analysis_id))
            traceback.print_exc()
            [api.dataset_abort_upload(target_id) for target_id in self.target_ids]
            
        api.queued_analysis_mark_ended(self.queued_analysis_id)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['behaviorcloud', 'behaviorcloud.analyzer']

package_data = \
{'': ['*']}

install_requires = \
['deepdiff>=4.3.2,<5.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'requests==2.23.0',
 'sentry-sdk>=0.14.4,<0.15.0']

setup_kwargs = {
    'name': 'bc-analyzer',
    'version': '0.2.3',
    'description': 'A package to create a BehaviorCloud compatible data analyzer.',
    'long_description': '# BehaviorCloud Analyzer\n#### Python Helper\n\n[![N|Solid](https://behaviorcloud.com/images/section/logo-4ffacbfa.png)](https://behaviorcloud.com/images/section/logo-4ffacbfa.png)\n\nThis is a thin package to help you quickly create a BehaviorCloud analyzer. It will handle calls to the BehaviorCloud API server so you don\'t have to worry about it. This package supports the following features: \n  - Fully managed analyzer creation - you just make the conversion function!\n  - One off analysis (pass an "--id" command-line parameter)\n  - Daemon analysis (use the "--daemon" flag)\n  - Exception handling and automatic upload to Sentry.io\n\nExample Usage:\n```python\nimport json\n\nfrom behaviorcloud.analyzer.data import convert_stream_to_json\nfrom behaviorcloud.analyzer.coordinator import Coordinator\n\ndef convert(source_request, source_settings, settings, source, targets):\n    target = targets[0]\n    source_data = convert_stream_to_json(source_request)\n    analyzed_data = [{processed: True, original: entry} for entry in source_data]\n    return [{\n        "data": json.dumps(analyzed_data),\n        "extension": "json",\n        "id": target["id"],\n    }]\n\ncoordinator = Coordinator(convert)\ncoordinator.run()\n```\n',
    'author': 'Christian Lent',
    'author_email': 'christian@behaviorcloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/behaviorcloud/bc-analyzer-py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import requests

from . import globals


def get_queued_analysis(id):
    response = requests.get(
        globals.get_full_url("queuedanalysis/%s" % (id)), headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()


def get_queued_analysis_index(path: str, ended=None, public=None):
    relative_url = "queuedanalysis?path=%s" % (path)
    if ended:
        relative_url += "&ended=1"
    if public:
        relative_url += "&public=1"
    response = requests.get(globals.get_full_url(relative_url), headers=globals.get_headers(),)
    response.raise_for_status()
    return response.json()


def queued_analysis_mark_started(id):
    response = requests.put(
        globals.get_full_url("queuedanalysis/%s/mark_started" % (id)),
        headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()


def queued_analysis_mark_ended(id):
    response = requests.put(
        globals.get_full_url("queuedanalysis/%s/mark_ended" % (id)), headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()


def get_dataset(id):
    response = requests.get(
        globals.get_full_url("datasets/%s" % (id)), headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()


def dataset_attach_data(id, data, extension):
    response = requests.put(
        globals.get_full_url("datasets/%s/attach_data" % (id)),
        json={"data": data, "extension": extension},
        headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()


def dataset_presign_upload(id):
    response = requests.post(
        globals.get_full_url("datasets/%s/presign_upload" % (id)),
        json={},
        headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()


def dataset_update_state(id, fields):
    response = requests.put(
        globals.get_full_url("datasets/%s/update_state" % (id)),
        json=fields,
        headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()


def dataset_abort_upload(id):
    response = requests.put(
        globals.get_full_url("datasets/%s/abort_upload" % (id)), headers=globals.get_headers(),
    )
    response.raise_for_status()
    return response.json()
